##
# This is a main Form Object class
class RFO::Base
  include Virtus.model

  if Rails.version > '4.1'
    include ActiveModel::Model
  else
    extend ActiveModel::Naming
    include ActiveModel::Conversion
    include ActiveModel::Validations
  end

  ##
  # Constructor:
  # ==== Arguments
  # * hash - where key is a name and value is an object. If key is an AR object or Draper object then key name will become instance variable with value defined in hash value.
  #
  # Run also +assign_defaults+ method to set default values
  def initialize(*args)
    options = args.extract_options!
    options.each_pair do |name, obj|
      next unless active_record_object?(obj)
      instance_variable_set("@#{name}", obj)
      super obj.attributes.symbolize_keys.slice(*self.attributes.keys)
    end
    assign_defaults
  end

  ##
  # We can specify Form Object to act as some class
  #
  # ==== Example
  #
  # # Form Object
  # class SomeForm < RFO::Base
  #   model_class Foo
  #
  #   attribute :name, String
  # end
  #
  # # In view we can use it to generate default paths in form.
  # = simple_form_for @some_form do |f|
  #  = f.input :name
  #  = f.submit
  #
  def self.model_class(name)
    @model_name = name || self
  end

  ##
  # Required for defining paths in rails
  #
  def self.model_name
    @model_name ||= self
    ActiveModel::Name.new(@model_name, nil)
  end

  def persisted?
    false
  end

  def save
    if valid?
      persist!
    else
      false
    end
  end

  def update_attributes(attributes_hash)
    self.attributes = attributes_hash
    self.save
  end

  protected
  ##
  # Method which will be overwritten in subclasses
  #
  def assign_defaults
  end

  private
  def active_record_object?(record)
    # Pure AR Object
    return true if pure_active_record?(record)

    # Decorated Object
    return true if record.respond_to?(:source) && pure_active_record?(record.source)

    false
  end

  def pure_active_record?(record)
    record.class.ancestors.include?(ActiveRecord::Base)
  end
end
